// gluGeometry class
// -Kyoung Shin Park (2016 Spring)
//////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <GL/glu.h>
#include "gluGeometry.h"

namespace moglclass
{

gluGeometry::gluGeometry(int quadricType)
{
	radius_ = 1.0;
	topRadius_ = 1.0;
	innerRadius_ = 0.0;
	height_ = 1.0;
	startAngle_ = 0.0;
	sweep_ = 180.0;
	slices_ = stacks_ = 8;
	rings_ = 1;
	axis_ = AXIS_Z;
	setUseNormals(GL_TRUE);
	setUseTexture(GL_TRUE);
	setDrawStyle(GLU_FILL);
	init(quadricType);
}

gluGeometry::~gluGeometry(void)
{
	gluDeleteQuadric(quadric_);
}

void gluGeometry::init(int quadricType)
{
	quadricType_ = quadricType;
	if (quadricType_ == GLU_SPHERE)
		makeSphere(radius_, slices_, stacks_);
	else if (quadricType_ == GLU_CYLINDER)
		makeCylinder(radius_, topRadius_, height_, slices_, stacks_);
	else if (quadricType_ == GLU_DISK)
		makeDisk(innerRadius_, radius_, slices_, rings_);
	else if (quadricType_ == GLU_PARTIAL_DISK)
		makePartialDisk(innerRadius_, radius_, slices_, rings_, startAngle_, sweep_);
	quadric_ = gluNewQuadric();
}

void gluGeometry::setUseNormals(GLboolean state)
{
	useNormals_ = state;
}

void gluGeometry::setUseTexture(GLboolean state)
{
	useTexture_ = state;
}

void gluGeometry::setDrawStyle(GLenum style)
{
	drawStyle_ = style;
}

void gluGeometry::makeSphere(GLdouble radius,GLint slices,GLint stacks)
{
	quadricType_ = GLU_SPHERE;
	radius_ = radius;
	slices_ = slices;
	stacks_ = stacks;
}

void gluGeometry::makeCylinder(GLdouble baseRadius, GLdouble topRadius,
                          GLdouble height, GLint slices, GLint stacks, int axis)
{
	quadricType_ = GLU_CYLINDER;
	radius_ = baseRadius;
	topRadius_ = topRadius;
	height_ = height;
	slices_ = slices;
	stacks_ = stacks;
	axis_ = axis;
}

void gluGeometry::makeCone(GLdouble baseRadius, GLdouble height,
                          GLint slices, GLint stacks,int axis)
{
	makeCylinder(baseRadius, 0.0, height, slices, stacks, axis);
}

void gluGeometry::makeDisk(GLdouble innerRadius, GLdouble outerRadius,
                          GLint slices, GLint rings,int axis)
{
	quadricType_ = GLU_DISK;
	radius_ = outerRadius;
	innerRadius_ = innerRadius;
	slices_ = slices;
	rings_ = rings;
	axis_ = axis;
}

void gluGeometry::makePartialDisk(GLdouble innerRadius, GLdouble outerRadius,
                          GLint slices, GLint rings,
                          GLdouble startAngle, GLdouble sweep,int axis)
{
	quadricType_ = GLU_PARTIAL_DISK;
	radius_ = outerRadius;
	innerRadius_ = innerRadius;
	slices_ = slices;
	rings_ = rings;
	startAngle_ = startAngle;
	sweep_ = sweep;
	axis_ = axis;
}

void gluGeometry::draw(void)
{
	gluQuadricTexture(quadric_, useTexture_);
	gluQuadricNormals(quadric_, useNormals_);
	gluQuadricDrawStyle(quadric_, drawStyle_);

	if (quadricType_ == GLU_SPHERE)
		gluSphere(quadric_, radius_, slices_, stacks_);
	else if (quadricType_ == GLU_CYLINDER)
		gluCylinder(quadric_, radius_, topRadius_, height_, slices_, stacks_);
	else if (quadricType_ == GLU_DISK)
		gluDisk(quadric_, innerRadius_, radius_, slices_, rings_);
	else if (quadricType_ == GLU_PARTIAL_DISK)
		gluPartialDisk(quadric_, innerRadius_, radius_, slices_, rings_, startAngle_, sweep_);
}

} // namespace
