#include "texture1D.h"
#include "simage.h"

namespace moglclass
{

Texture1D::Texture1D(char *filename, GLint wrap, GLint min, GLint mag) :
                Texture(wrap,min,mag)
{
	uiTexture = 0;
	uiSampler = 0;

    imageData = NULL;
    rescaledImageData = NULL;
    loaded = false;
    createdImage = false;
    if (filename)
    {
		imageData = simage_read_image(filename, &width, &height, &numComponents);
		createdImage = true;
    }
	load();
}

Texture1D::~Texture1D(void)
{
    if (rescaledImageData)
        simage_free_image(rescaledImageData);
    if (createdImage)
        simage_free_image(imageData);

	glDeleteTextures(1, &uiTexture);
	glDeleteSamplers(1, &uiSampler);
}

void Texture1D::load(void)
{
	GLenum type;
    GLsizei xdim2;
    unsigned char *imgPtr;
    loaded = true;

	if (!imageData)
        return;
    xdim2 = 1;
    while (xdim2 <= width)
        xdim2 *= 2;
    xdim2 /= 2;
    if (width != xdim2)
        {
        rescaledImageData = simage_resize(imageData, width, height, numComponents,
                                           xdim2, 1);
        imgPtr = rescaledImageData;
        }
    else
        imgPtr = imageData;

	if (numComponents == 4) type = GL_RGBA;
	else if (numComponents == 3) type = GL_RGB;
	else if (numComponents == 2) type = GL_LUMINANCE_ALPHA;
	else if (numComponents == 1) type = GL_LUMINANCE;

	if (uiTexture == 0)
        glGenTextures(1, &uiTexture);

    glBindTexture(GL_TEXTURE_1D, uiTexture);
    if ((minFilter == GL_NEAREST_MIPMAP_NEAREST) ||
          (minFilter == GL_NEAREST_MIPMAP_LINEAR) ||
          (minFilter == GL_LINEAR_MIPMAP_NEAREST) ||
          (minFilter == GL_LINEAR_MIPMAP_LINEAR))
        gluBuild1DMipmaps(GL_TEXTURE_1D, (GLint) type, xdim2, type,
                          GL_UNSIGNED_BYTE, imgPtr);
    else
        glTexImage1D(GL_TEXTURE_1D, 0, (GLint) type, xdim2, 0, type,
                     GL_UNSIGNED_BYTE, imgPtr);
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MAG_FILTER, magFilter);
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER, minFilter);
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S, wrapS);
    glBindTexture(GL_TEXTURE_1D,0); // switch to default texture object

	if (uiSampler == 0)
		glGenSamplers(1, &uiSampler);
	// Set magnification filter
	if(magFilter == TEXTURE_FILTER_MAG_NEAREST)
		glSamplerParameteri(uiSampler, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	else if(magFilter == TEXTURE_FILTER_MAG_BILINEAR)
		glSamplerParameteri(uiSampler, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	// Set minification filter
	if(minFilter == TEXTURE_FILTER_MIN_NEAREST)
		glSamplerParameteri(uiSampler, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	else if(minFilter == TEXTURE_FILTER_MIN_BILINEAR)
		glSamplerParameteri(uiSampler, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	else if(minFilter == TEXTURE_FILTER_MIN_NEAREST_MIPMAP)
		glSamplerParameteri(uiSampler, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
	else if(minFilter == TEXTURE_FILTER_MIN_BILINEAR_MIPMAP)
		glSamplerParameteri(uiSampler, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	else if(minFilter == TEXTURE_FILTER_MIN_TRILINEAR)
		glSamplerParameteri(uiSampler, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	// Set wraping
	glSamplerParameteri(uiSampler, GL_TEXTURE_WRAP_S, wrapS);
}

void Texture1D::bind()
{
	if (!loaded) return;

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, uiTexture);
	//glUniform1i(uiSamplerID, 0);// set sampler
	glBindSampler(0, uiSampler);// set sampler state
}

void Texture1D::disable(void)
{
    glBindTexture(GL_TEXTURE_1D,0);
    glDisable(GL_TEXTURE_1D);
}

void Texture1D::setSamplerParameter(GLenum parameter, GLenum value)
{
	glSamplerParameteri(uiSampler, parameter, value);
}

} // namespace
